// 2025/03/22
// Rx : SAMD21(1.8.5), RP2040(4.5.0), C3(2.0.17, 3.1.3), S3(2.0.17, 3.1.3), C6(3.1.3), nRF52840(1.1.9), MG24(2.3.0)
//      [NOTE] Compile ERROR : nRF52840(2.9.3)
//      [NOTE] RP2040 not work U8g2 display
// Tx : SAMD21(1.8.5)

#include <U8g2lib.h>        // U8g2 2.35.30
#include <RadioLib.h>       // RadioLib 7.1.2

#define TX_UID 0x1234ABCD   // TX device UID example
#define RF_SW	D5			      // RF switch 

//#define LED_BUILTIN D0      // for ESP32C3

// SSD1306 software I2C library, SCL=D6, SDA=D7, 400kHz
U8G2_SSD1306_128X64_NONAME_F_SW_I2C display(U8G2_R0, /*clock=*/ D6, /*data=*/ D7, /*reset=*/ U8X8_PIN_NONE);

// Radio lib Modele default setting : SPI:2MHz, MSBFIRST, MODE0
SX1262 radio = new Module(/*NSS*/D4, /*DIO1*/D1, /*NRST*/D2, /*BUSY*/D3);

#define dataNum   12                // data bytes
union unionData {                   // for data type conversion
  uint32_t  dataBuff32[dataNum/4];
  float     dataBuff32F[dataNum/4];
  int16_t   dataBuff16[dataNum/2];
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;

int rssi;               // signal RSSI [dBm]
int snr;                // signal SN ratio [dB]
int bps;                // data rate [bps]
uint32_t txuid;         // TX Device UID
uint16_t sec;           // mS/1000 data [sec]
float Vbatt;            // battery voltage data [V]
char printBuff[4];      // for sprintf()
String  txdata = "";    // transmission data packet string
String  rxdata = "";    // received data packet string
bool rxstatus = false;  // receive status flag

// *******************************************************************************************************
void setup() {
  Serial.begin(115200);
//  while(!Serial);
  delay(2000);
  Serial.println("Wio-SX1262 for XIAO LoRa TEST");

  // Pin initialization
  pinMode(RF_SW, OUTPUT);				        // RF switch
  digitalWrite(RF_SW, HIGH);			      // internal DIO2 controls Tx/Tx
  pinMode(LED_BUILTIN, OUTPUT);         // built-in LED
  digitalWrite(LED_BUILTIN, HIGH);      // LED off

  // int16_t begin(float freq = 434.0, float bw = 125.0, uint8_t sf = 9, uint8_t cr = 7, \
  // uint8_t syncWord = RADIOLIB_SX126X_SYNC_WORD_PRIVATE, int8_t power = 10, \
  // uint16_t preambleLength = 8, float tcxoVoltage = 1.6, bool useRegulatorLDO = false);
  int state = radio.begin(922.6, 125.0, 12, 5, 0x12, 13, 8, 1.7, 0);

  if (state == RADIOLIB_ERR_NONE) {
    Serial.println("radio.begin() success!");
  }
  else {
    Serial.print("failed, code ");
    Serial.println(state);
    while(true) {
      digitalWrite(LED_BUILTIN, LOW);
      delay(100);
      digitalWrite(LED_BUILTIN, HIGH);
      delay(100);      
    }
  }

  // Display initialization
  display.begin();
  display.clearDisplay();
  display.setFont(u8g2_font_8x13B_tr);
  display.setCursor(0, 15);
  display.println("Wio-X1262");
  display.setCursor(0, 31);
  display.println("P2P Test");
  display.setCursor(0, 47);
  display.println("RX");
  display.sendBuffer();  
  delay(2000);
  display.clearDisplay();   

  // setup end 
  display.setCursor(0, 47);
  display.print("SX1262 RX");
  display.setCursor(0, 63);
  display.print("Setup END");
  display.sendBuffer();
  delay(2000);
  display.clearDisplay();
   
}

// ***********************************************************************************************************
void loop() 
{
  int state;
  Serial.println("Waiting for incoming Slave transmission ... ");

  do {
    // received error status
    rxstatus = false;

    // receive
    digitalWrite(LED_BUILTIN, LOW);
    state = radio.receive(ud.dataBuff8, dataNum);
    digitalWrite(LED_BUILTIN, HIGH);
//    txuid = ud.dataBuff32[0];          // TX device UID

    if (state != RADIOLIB_ERR_NONE) {
      // reseived error, error status check
      if (state == RADIOLIB_ERR_RX_TIMEOUT) {
      // timeout occurred while waiting for a packet
      Serial.println("RX timeout!");
      } 
      else if (state == RADIOLIB_ERR_CRC_MISMATCH) {
      // packet was received, but is malformed
      Serial.println("CRC error!");
      }
    } 
    else {
      // received success, TX_UID check
      txuid = ud.dataBuff32[0];          // TX device UID
      if (txuid != TX_UID) {
        Serial.print("TX_UID mismatch ");
        Serial.println(txuid, HEX);
      }
      else {
        // received success status
        rxstatus = true;        
      }
    }
  } while(rxstatus == false);

  // packet was successfully received
  Serial.print("RX success! ");
  Serial.println(txuid, HEX);

  // print the data of the packet
  rxdata = "";
  for(int i = 0; i < dataNum; i++) {
    sprintf(printBuff, "%02x", ud.dataBuff8[i]);
    rxdata += printBuff;
  }
  rxdata.toUpperCase();
  Serial.println(rxdata);

  // restoration of received data
  sec = ud.dataBuff32[1] / 1000;      // transmission time
  Vbatt = ud.dataBuff32F[2];          // battery voltage

  Serial.println(sec);
  Serial.println(Vbatt, 3);

  // print the RSSI (Received Signal Strength Indicator)
  // of the last received packet
  Serial.print("RSSI:\t\t");
  rssi = radio.getRSSI();
  Serial.print(rssi);
  Serial.println(" dBm");

  // print the SNR (Signal-to-Noise Ratio)
  // of the last received packet
  Serial.print("SNR:\t\t");
  snr = radio.getSNR();
  Serial.print(snr);
  Serial.println(" dB");

  display.clearDisplay();
  display.setCursor(0, 15); display.print("from");
  display.setCursor(64, 15); display.print(TX_UID, HEX);  
  display.setFont(u8g2_font_5x8_tr);
  display.setCursor(0, 31); display.print(rxdata);
  display.setFont(u8g2_font_8x13B_tr);
  display.setCursor(0, 47); display.print(rssi); display.print("dBm");
  display.setCursor(64, 47); display.print(snr); display.print("dB");
  display.setCursor(0, 63); display.print(sec); display.print("sec");
  display.setCursor(64, 63); display.print(Vbatt, 3); display.print("V");   
  display.sendBuffer();
   
}

