// 2025/03/22
// Tx : SAMD21(1.8.5), RP2040(4.5.0), C3(2.0.17, 3.1.3), S3(2.0.17, 3.1.3), C6(3.1.3), nRF52840(1.1.9), MG24(2.3.0)
//      [NOTE] Compile ERROR : nRF52840(2.9.3), MG24(2.3.0)
// Rx : SAMD21(1.8.5)

#include <RadioLib.h>           // RadioLib 7.1.2

#define TX_UID  0x1234ABCD;		  // TX device UID example
#define RF_SW	  D5				      // RF switch

//#define LED_BUILTIN D0        // for ESP32C3

// Radio lib Modele default setting : SPI:2MHz, MSBFIRST, MODE0
SX1262 radio = new Module(/*NSS*/D4, /*DIO1*/D1, /*NRST*/D2, /*BUSY*/D3);

#define dataNum   12                // data bytes
union unionData {                   // for data type conversion
  uint32_t  dataBuff32[dataNum/4];
  float     dataBuff32F[dataNum/4];
  uint16_t  dataBuff16[dataNum/2];
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;

int bps;                // data rate [bps]
uint32_t uid = TX_UID;  // TX device UID
uint16_t sec;           // mS/1000 data [sec]
float Vbatt;            // battery voltage data [V]
char printBuff[4];      // for sprintf()
String  txdata = "";    // transmission data packet string

// *******************************************************************************************************
void setup() {
  Serial.begin(115200);
//  while(!Serial);
  delay(2000);

  Serial.println("Wio-SX1262 for XIAO LoRa TEST");

  // Pin initialization
  pinMode(RF_SW, OUTPUT);				        // RF switch
  digitalWrite(RF_SW, HIGH);			      // internal DIO2 controls Tx/Tx
  pinMode(LED_BUILTIN, OUTPUT);         // built-in LED
  digitalWrite(LED_BUILTIN, HIGH);      // LED off

  // int16_t begin(float freq = 434.0, float bw = 125.0, uint8_t sf = 9, uint8_t cr = 7, \
  // uint8_t syncWord = RADIOLIB_SX126X_SYNC_WORD_PRIVATE, int8_t power = 10, \
  // uint16_t preambleLength = 8, float tcxoVoltage = 1.6, bool useRegulatorLDO = false);
  int state = radio.begin(922.6, 125.0, 12, 5, 0x12, 13, 8, 1.7, 0);

  if (state == RADIOLIB_ERR_NONE) {
    Serial.println("radio.begin() success!");
  }
  else {
    Serial.print("failed, code ");
    Serial.println(state);
    while(true) {
      digitalWrite(LED_BUILTIN, LOW);
      delay(100);
      digitalWrite(LED_BUILTIN, HIGH);
      delay(100);      
    }
  }
}

// ***********************************************************************************************************
void loop() 
{
  Serial.println("");
  Serial.println("Transmitting packet ... ");

  // transmits data
  ud.dataBuff32[0] = uid;                         // TX device UID
  ud.dataBuff32[1] = millis();                    // transmission time
  Vbatt = 3.789; // example
  ud.dataBuff32F[2] = Vbatt;                      // battery voltage
  sec = ud.dataBuff32[1] / 1000;                  // transmission time sec

  digitalWrite(LED_BUILTIN, LOW);
  int state = radio.transmit(ud.dataBuff8, dataNum);
  digitalWrite(LED_BUILTIN, HIGH);

  if (state == RADIOLIB_ERR_NONE) {
    // the packet was successfully transmitted
    Serial.println("TX success!");

    // print the packet
    txdata = "";
    for(int i = 0; i < dataNum; i++) {
      sprintf(printBuff, "%02x", ud.dataBuff8[i]);
      txdata += printBuff;
    }
    txdata.toUpperCase();
    Serial.println(txdata);

    Serial.println(ud.dataBuff32[0], HEX);
    Serial.println(ud.dataBuff32[1]);
    Serial.println(ud.dataBuff32F[2], 3);

    // print measured data rate
    Serial.print("Datarate:\t");
    bps = radio.getDataRate();
    Serial.print(bps);
    Serial.println(" bps");
  }

  // error status check
  else if (state == RADIOLIB_ERR_PACKET_TOO_LONG) {
    // the supplied packet was longer than 256 bytes
    Serial.println("too long!");
  }
  else if (state == RADIOLIB_ERR_TX_TIMEOUT) {
    // timeout occured while transmitting packet
    Serial.println("TX timeout!");
  }
  else {
    // some other error occurred
    Serial.print("failed, code ");
    Serial.println(state);
  }
  delay(2000); 
}
